/**
 * @file          /ks-license/lib/base/license-error.cpp
 * @brief
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 */

#include "lib/base/license-error.h"

#include <fmt/format.h>
#include <glib/gi18n.h>

namespace Base
{
LicenseError::LicenseError()
{
}

std::string LicenseError::get_error_desc(LicenseErrorCode error_code)
{
    std::string error_desc;
    switch (error_code)
    {
    case LicenseErrorCode::LEC_COMMON_UNSUPPORTED_OPERATION:
        error_desc = _("Unsupported operation.");
        break;
    case LicenseErrorCode::LEC_COMMON_IO_ERROR:
        error_desc = _("IO Error.");
        break;
    case LicenseErrorCode::LEC_LICENSE_LENGTH_INVALID:
    case LicenseErrorCode::LEC_LICENSE_TRANS_FAILED:
    case LicenseErrorCode::LEC_LICENSE_DECODE_FAILED:
    case LicenseErrorCode::LEC_LICENSE_DATE_INVALID:
    case LicenseErrorCode::LEC_UNMATCH_MACHINE_CODE:
    case LicenseErrorCode::LEC_CUSTOMER_INFO_INVALID:
        error_desc = _("Activation code error.");
        break;
    case LicenseErrorCode::LEC_INSTALL_TYPE_UNMATCH_MACHINE:
        error_desc = _("The activation code does not match the machine.");
        break;
    case LicenseErrorCode::LEC_LICENSE_EXPIRED:
        error_desc = _("The activation code is expired.");
        break;
    case LicenseErrorCode::LEC_MACHINE_CODE_TRANS_FAILED:
        error_desc = _("Machine code error.");
        break;
    case LicenseErrorCode::LEC_SYNC_TO_FILE_FAILED:
        error_desc = _("Failed to write activation code.");
        break;
    case LicenseErrorCode::MACHINE_CODE_UNMATCH_INSTALL_TYPE:
        error_desc = _("The install type does not match the machine code.");
        break;
    case LicenseErrorCode::LEC_LICENSE_ACTIVATION_CODE_TYPE_ERROR:
        error_desc = _("The type of activation code is invalid.");
        break;
    case LicenseErrorCode::LEC_LICENSE_NOT_SUPPORT_ONLINE_ACTIVATION:
        error_desc = _("Online activation is not supported.");
        break;
    case LicenseErrorCode::LEC_UNACTIVE_OS_FORBID_ACTIVE_VA:
        error_desc = _("Active value-added service must active KylinsecOS first.");
        break;
    case LicenseErrorCode::LEC_OS_LICENSE_EXPIRY_FORBID_ACTIVE_VA:
        error_desc = _("KylinsecOS has expired, active value-added service must purchase the technical service again.");
        break;

    case LicenseErrorCode::LEC_UKEY_NOT_FOUND:
        error_desc = _("USBKey device is not found, please insert USBKey device.");
        break;
    case LicenseErrorCode::LEC_UKEY_NOT_USABLE_UKEY_DEVICE:
        error_desc = _("There is no USBKey device available. Please check the expiration date and remaining points of USBKey device.");
        break;
    case LicenseErrorCode::LEC_UKEY_UNSUPPORTED_ARCH:
        error_desc = _("The device is not supported by the system architecture.");
        break;
    case LicenseErrorCode::LEC_CONNECT_SERVER_FAILED:
        error_desc = _("Failed to connect license server.");
        break;
    case LicenseErrorCode::LEC_SEND_ACTIVATION_REQUEST_FAILED:
        error_desc = _("Failed to send activation request.");
        break;
    case LicenseErrorCode::LEC_SERVER_RESPONSE_TIMEOUT:
        error_desc = _("Server response timeout.");
        break;
    case LicenseErrorCode::LEC_SERVER_RESPONSE_ERROR:
        error_desc = _("Server response error.");
        break;
    case LicenseErrorCode::LEC_SERVER_SERVICE_UID_NOT_FOUND:
        error_desc = _("No matching service uid found.");
        break;
    case LicenseErrorCode::LEC_SERVER_ACTIVATION_RECORD_NOT_FOUND:
        error_desc = _("Not found activation record.");
        break;
    case LicenseErrorCode::LEC_SERVER_CS_TIME_DIFF_TOO_MUCH:
        error_desc = _("The time difference between server and client is too much.");
        break;
    case LicenseErrorCode::LEC_SERVER_MACHINE_TYPE_DISMATCH:
        error_desc = _("The machine type is dismatch.");
        break;
    case LicenseErrorCode::LEC_SERVER_IMPORT_POINTS_STATUS_EXPIRED:
        error_desc = _("Import points expired.");
        break;
    case LicenseErrorCode::LEC_SERVER_IMPORT_POINTS_ALREADY_EXISTS:
        error_desc = _("Service number already exists.");
        break;
    case LicenseErrorCode::LEC_UKEY_NO_REMAINING_POINTS:
    case LicenseErrorCode::LEC_SERVER_NO_REMAINING_POINTS:
        error_desc = _("No remaining points.");
        break;
    case LicenseErrorCode::LEC_SERVER_REQUEST_PAGE_NO_ERROR:
        error_desc = _("Page does not exist, please try refreshing.");
        break;
    case LicenseErrorCode::LEC_SERVER_ACTIVATION_OBJECT_MISMATCH:
        error_desc = _("Activation object mismatch.");
        break;
    case LicenseErrorCode::LEC_SERVER_INIT_POINTS_FAILED:
        error_desc = _("Service number init failed.");
        break;
    case LicenseErrorCode::LEC_SERVER_GENERATE_EXPIRY_TIME_FAILED:
        error_desc = _("Generate expiry time failed.");
        break;
    case LicenseErrorCode::LEC_SERVER_PERIOD_TYPE_INVALID:
        error_desc = _("Period type Invalid.");
        break;
    case LicenseErrorCode::LEC_SERVER_ALREADY_EXIST_MACHINE_CODE:
        error_desc = _("Machine code has exist.");
        break;
    case LicenseErrorCode::LEC_SERVER_NOT_EXIST_MACHINE_CODE:
        error_desc = _("Machine code can't exist.");
        break;
    case LicenseErrorCode::LEC_SERVER_ACTIVATION_RECORD_UPDATE_FAILED:
        error_desc = _("Update activation record failed.");
        break;
    case LicenseErrorCode::LEC_CUSTOMIZATION_CUSTOMER_NAME_MISMATCH:
        error_desc = _("Customization customer name mismatch.");
        break;
    case LicenseErrorCode::LEC_INPUT_DATE_INVALID:
    case LicenseErrorCode::LEC_INPUT_CUSTOMER_LENGTH_INVALID:
        error_desc = _("Input error.");
        break;
    case LicenseErrorCode::LEC_EXPIRED_DATE_LARGE:
    case LicenseErrorCode::LEC_UKEY_GROUP_UNINIT:
    case LicenseErrorCode::LEC_UKEY_EXPIRED_DATE_INVALIDE:
    case LicenseErrorCode::LEC_ENCODE_GENERATE_LICENSE_FAILED:
    case LicenseErrorCode::LEC_GENERATE_LICENSE_LENGTH_INALID:
    case LicenseErrorCode::LEC_SYNC_ACTIVATION_TO_DB_FAILED:
    case LicenseErrorCode::LEC_SYNC_POINTS_TO_DB_FAILED:
    case LicenseErrorCode::LEC_UKEY_ALREADY_EXIST_MACHINE_CODE:
    case LicenseErrorCode::LEC_UKEY_NOT_EXIST_MACHINE_CODE:
    case LicenseErrorCode::LEC_UKEY_NO_USED_POINTS:
    case LicenseErrorCode::LEC_C51_GENERATE_LICENSE_FAILED:
    case LicenseErrorCode::LEC_UKEY_ITEM_EXCEED_LIMIT:
    case LicenseErrorCode::LEC_UKEY_PIN_FAILED:
    case LicenseErrorCode::LEC_UKEY_WRITE_MACHINE_CODE_FAILED:
    case LicenseErrorCode::LEC_UKEY_WRITE_REMAINING_POINTS_FAILED:
        error_desc = _("Data error.");
        break;
    default:
        error_desc = _("Unknown error.");
    }

    error_desc += fmt::format(_(" (error code: 0x{:x})"), int32_t(error_code));
    return error_desc;
}
}  // namespace Base