/**
 * @file          /ks-license/src/daemon/dbus/dbus-license-helper.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */
#pragma once

#include "lib/base/base.h"
#include "license_helper_dbus_stub.h"
#include "src/daemon/license-object.h"

namespace Daemon
{
class DBusLicenseHelper : public com::kylinsec::Kiran::LicenseHelper_adaptor,
                          public DBus::IntrospectableAdaptor,
                          public DBus::ObjectAdaptor

{
public:
    DBusLicenseHelper(DBus::Connection& connection);
    virtual ~DBusLicenseHelper();

    void init();

    bool timing_check_license();

public:
    // 获取激活信息
    virtual ::DBus::Struct<uint16_t, std::string, std::string, uint16_t, uint32_t, uint32_t> GetLicense();
    // 获取激活信息，以json格式返回
    virtual std::string GetLicenseJson();
    // 获取服务状态
    virtual int32_t GetServiceStatus();
    // 获取机器码
    virtual std::string GetMachineCode();
    // 通过激活码激活
    virtual void RegisterByLicenseCode(const std::string& activation_code);
    // 通过UKey设备激活
    virtual std::string RegisterByUsbKey();
    // 在线激活
    virtual void RegisterOnLine(const std::string& host, const std::string& service_uid);

private:
    void on_license_info_changed();

private:
    std::shared_ptr<LicenseObject> license_object_;
    LicenseInfo license_info_;
    // 由于传递给DBus::Error的参数必须是不能被立即销毁的，因此将数据放入成员变量存储
    std::string dbus_error_message_;
};
}  // namespace Daemon
