#include "lib/base/base.h"

namespace Daemon
{
struct NetDeviceInfo;
struct CpuCoreInfo;

class HardwareControl
{
private:
    HardwareControl(/* args */);
    ~HardwareControl();

public:
    static void global_init();
    static void global_deinit();
    static HardwareControl *get_instance() { return instance_; };

    void limit_bandwidth(int bandwidth);
    void limit_cpu_core(int core_number);
    int get_cpu_core_number();

    bool is_cpu_core_limit();
    bool is_bandwidth_limit();

    void enable_all_cpu_core();
    void unlimit_all_net_device();

private:
    void init();

    // 设置CPU核心使能状态
    bool set_cpu_core_state(const std::string &cpu_core_name, bool enable);
    bool get_cpu_core_state(const std::string &cpu_core_name, bool &cpu_core_state);
    int get_disabled_cpu_number();

    int limit_net_device(const std::string &net_dev_name, int bandwidth);
    int get_net_device_bandwidth(const std::string &net_device_name);
    bool is_wifi_device(const std::string &device_name);

    // 读取cpu信息：cpu0将不被读取，不限制cpu0
    void read_cpu_core_info(std::vector<std::shared_ptr<CpuCoreInfo>> &cpu_core_info);
    void read_net_devices_info(std::vector<std::shared_ptr<NetDeviceInfo>> &net_devices);
    // 重置网络设备
    bool reset_net_device(const std::string &device_name);
    int get_net_device_max_bandwidth(const std::string &standard_output);

    std::vector<int> get_enabled_bandwidth(const std::string &standard_output);
    bool is_support_bandwidth(const std::string &device_name, int bandwidth);

private:
    static HardwareControl *instance_;
    std::vector<std::shared_ptr<CpuCoreInfo>> cpu_cores_info_;
    std::vector<std::shared_ptr<NetDeviceInfo>> net_devices_info_;
};

}  // namespace Daemon