/**
 * @file          /ks-license/src/daemon/license-object-os-pg.h
 * @brief
 * @author        xiaowen <xiaowen@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 */

#pragma once

#include "hardware-control.h"
#include "license-object-os.h"
#include "yum-repo-control.h"

namespace Daemon
{
class LicenseObjectOSPG : public LicenseObjectOS
{
public:
    LicenseObjectOSPG(const std::string& name);
    ~LicenseObjectOSPG();

private:
    void expiry_limit();
    void serious_expiry_limit();
    void on_activation_success();

    void unlimit_hardware();
    void on_check_license_state();
    void update_certificate_file();
    bool is_updating_certificate();
    bool timing_check_certificate();

    void on_unlimit_file_changed(const Glib::RefPtr<Gio::File>& file,
                                 const Glib::RefPtr<Gio::File>& other_file,
                                 Gio::FileMonitorEvent event_type);
    // 检测取消限制文件有效性
    bool check_unlimit_file_validity();

private:
    HardwareControl* hardware_control;
    std::shared_ptr<sigc::signal<void>> expiry_signal_;
    std::shared_ptr<sigc::signal<void>> serious_expiry_signal_;
    std::shared_ptr<YumRepoControl> yum_repo_control_;
    sigc::connection certificate_timer_;
    Glib::RefPtr<Gio::FileMonitor> unlimit_monitor_;
};

}  // namespace Daemon