/**
 * @file          /ks-license/src/daemon/license-object-os.cpp
 * @brief
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 */

#include "src/daemon/license-object-os.h"
#include <fcntl.h>
#include <sys/ioctl.h>
#include <fstream>

namespace Daemon
{
LicenseObjectOS::LicenseObjectOS(const std::string& name) : LicenseObject(name)
{
    auto timeout = Glib::MainContext::get_default()->signal_timeout();
    this->status_check_timer_ = timeout.connect_seconds(sigc::mem_fun(this, &LicenseObjectOS::timing_check_status),
                                                        // LICENSE_CHECK_INTERVAL);
                                                        120);
}

LicenseObjectOS::~LicenseObjectOS()
{
}
bool LicenseObjectOS::timing_check_status()
{
    KLOG_PROFILE("");

    auto license_info = this->get_license_info();
    auto expired_time = time(NULL) - license_info.expired_time;

    std::shared_ptr<sigc::signal<void>> state_signal;

    for (auto iter = this->license_status_.begin(); iter != this->license_status_.end(); iter++)
    {
        if (iter->first * DAY_IN_SECONDS <= expired_time)
        {
            state_signal = iter->second;
        }
    }

    if (state_signal)
    {
        state_signal->emit();
    }
    else
    {
        this->check_license_state_.emit();
    }

    return true;
}

void LicenseObjectOS::regist_license_state(int days, std::shared_ptr<sigc::signal<void>> signal)
{
    std::pair<int, std::shared_ptr<sigc::signal<void>>> opration(days, signal);
    this->license_status_.insert(opration);
}

}  // namespace Daemon
