/**
 * @file          /kylin-license/src/daemon/ukey-manager.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */
#pragma once

#include "src/daemon/ukey-device.h"

namespace Daemon
{
class UKeyManager
{
public:
    virtual ~UKeyManager();

    static UKeyManager* get_instance() { return instance_; };

    static void global_init();

    static void global_deinit() { delete instance_; };

    // 获取一个可用于激活机器码的ukey设备
    std::shared_ptr<UKeyDevice> get_ukey(const std::string& activation_object_name);
    // 获取一个可用于激活机器码的ukey设备,优先查找已经记录machine_code_str的设备
    std::shared_ptr<UKeyDevice> get_ukey(const std::string& activation_object_name, const std::string& machine_code_str);

private:
    UKeyManager();

    void init();
    void load_physical_ukeys();
    void clear_physical_ukeys();

    void add_ukey(std::shared_ptr<UKeyDevice> ukey_device);

    // 获取机器码所在的UKey设备
    std::shared_ptr<UKeyDevice> get_ukey_by_machine_code(const std::string& machine_code);

    // 这里使用空指针，避免在头文件中包含gudev.h文件
    static void on_uevent_changed(void* client, gchar* action, void* device, gpointer user_data);

private:
    static UKeyManager* instance_;
    void* udev_client_;
    uint64_t udev_client_handler_;
    // <硬件序列号, USBKey数据>
    std::map<std::string, std::shared_ptr<UKeyDevice>> ukeys_;
};

}  // namespace Daemon