#pragma once
#include "src/daemon/license-object.h"

typedef struct _SoupSession SoupSession;

namespace Daemon
{
#define DEFAULT_VALIDITY_PERIOD 7
class YumRepoControl
{
public:
    YumRepoControl(const std::string &repofile_name);
    ~YumRepoControl();

    // 下载证书
    bool download_certificate_file();

    sigc::signal<void> signal_certificate_file_deleted() { return this->certificate_file_deleted_; }
    void insert_repo_group(std::string repo_group) { this->repo_group_names_.push_back(repo_group); }
    void enable_repo_groups(std::string repo_name, bool enable);

private:
    // 加载dnf变量
    void load_dnf_vars();

    // 是否可以访问证书服务器
    bool ping_certificate_server();

    void on_dnf_vars_changed(const Glib::RefPtr<Gio::File> &file,
                             const Glib::RefPtr<Gio::File> &other_file,
                             Gio::FileMonitorEvent event_type);

    void on_certificate_file_changed(const Glib::RefPtr<Gio::File> &file,
                                     const Glib::RefPtr<Gio::File> &other_file,
                                     Gio::FileMonitorEvent event_type);
    bool generate_certificate_file();

    void enable_repo_groups(bool enable);

private:
    SoupSession *http_session_;
    std::string repo_file_name_;

    Glib::RefPtr<Gio::FileMonitor> dnf_vars_monitor_;
    Glib::RefPtr<Gio::FileMonitor> certificate_crt_monitor_;
    Glib::RefPtr<Gio::FileMonitor> certificate_key_monitor_;
    std::string osversion_;
    std::string arch_;
    sigc::signal<void> certificate_file_deleted_;
    std::vector<std::string> repo_group_names_;
};

}  // namespace Daemon