/**
 * @file          /ks-license/src/server/license-impl.h
 * @brief
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 */

#include <arpa/inet.h>
#include <ifaddrs.h>
#include <license.grpc.pb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include "lib/base/str-utils.h"
#include "src/server/license-database.h"
#include "src/server/license-points-manager.h"

namespace Server
{
class LicenseImpl : public License::Service
{
public:
    LicenseImpl();
    virtual ~LicenseImpl();

    virtual ::grpc::Status GetActivationCode(::grpc::ServerContext* context, const ::ActivationReq* request, ::ActivationRsp* response);
    virtual ::grpc::Status HeartBeat(::grpc::ServerContext* context, const ::HeartBeatReq* request, ::HeartBeatRsp* response);
    virtual ::grpc::Status GetPointsRecords(::grpc::ServerContext* context, const ::PointsRecordsReq* request, ::PointsRecordsRsp* response);
    virtual ::grpc::Status GetActivationRecords(::grpc::ServerContext* context, const ::ActivationRecordReq* request, ::ActivationRecordsRsp* response);
    virtual ::grpc::Status ImportPointsRecords(::grpc::ServerContext* context, const ::ImportPointsReq* request, ::ImportPointsRsp* response);
    virtual ::grpc::Status GetServerInfo(::grpc::ServerContext* context, const ::ServerInfoReq* request, ::ServerInfoRsp* response);
    virtual ::grpc::Status GetActivationRecordsByCond(::grpc::ServerContext* context, const ::ActivationRecordsByCondReq* request, ::ActivationRecordsByCondRsp* response);
    virtual ::grpc::Status UpdatePointsRecords(::grpc::ServerContext* context, const ::UpdatePointsReq* request, ::UpdatePointsRsp* response);
    virtual ::grpc::Status DeletePointsRecords(::grpc::ServerContext* context, const ::DeletePointsReq* request, ::DeletePointsRsp* response);

private:
    // 填充激活记录信息
    void fill_activation_records(::ActivationRecord* activation_record_pb, std::shared_ptr<ActivationRecord> activation_record);
    void update_activations_cache(const std::string& keywords);
    // 模糊搜索激活信息
    bool fuzzy_search_activation_info(const std::map<std::string, std::string>& activation_info, const std::string& keywords);
    // 精确搜索激活信息：keywords格式：$key=value
    bool accurate_search_activation_info(const std::map<std::string, std::string>& activation_info, const std::string& keywords);
    // 获取当前机器码
    std::string get_machine_code();

private:
    LicensePointsManager* points_manager_;
    std::shared_ptr<LicenseDatabase> database_;
    std::mutex mutex_;

    ActivationRecordVec activations_cache_;
    std::shared_ptr<Glib::KeyFile> license_config_;
};

}  // namespace Server
