/**
 * @file          /ks-license/src/server/license-points-manager.h
 * @brief
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved.
 */

#pragma once

#include "src/server/license-database.h"
#include "src/server/license-points.h"

namespace Server
{
class LicensePointsManager
{
public:
    virtual ~LicensePointsManager();

    static LicensePointsManager* get_instance() { return instance_; };

    static void global_init();

    static void global_deinit() { delete instance_; };

    // 获取点数记录
    LicensePointsVec get_license_pointss();
    std::shared_ptr<LicensePoints> get_license_points(const std::string& service_uid);

    bool import_points_file(const std::string& file_name, LicenseErrorCode& error_code);

    LicenseErrorCode insert_points(std::shared_ptr<PointsRecord> points_record);
    LicenseErrorCode update_points(const std::string& service_uid, const std::string& expiry_day);
    LicenseErrorCode update_points(const std::string& service_uid, int32_t warranty_days, int32_t warranty_months, int32_t warranty_years);
    LicenseErrorCode delete_points(const std::string& servise_uid);

private:
    LicensePointsManager();

    void init();
    void load_points();

    void add_license_points(std::shared_ptr<LicensePoints> license_points);

    LicenseErrorCode add_new_license_points(std::shared_ptr<PointsRecord> points_record);

    // 这里使用空指针，避免在头文件中包含gudev.h文件
    static void on_uevent_changed(void* client, gchar* action, void* device, gpointer user_data);

private:
    static LicensePointsManager* instance_;

    std::shared_ptr<LicenseDatabase> database_;
    // <序列号, 点数记录>
    std::map<std::string, std::shared_ptr<LicensePoints>> points_records_;
    std::mutex mutex_;
};

}  // namespace Server