/**
 * @file          /ks-license/src/server/main.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include <glib/gi18n.h>
#include <grpc/grpc.h>
#include <grpcpp/server.h>
#include <grpcpp/server_builder.h>
#include <grpcpp/server_context.h>
#include <locale.h>
#include "src/server/license-impl.h"
#include "src/server/license-points-manager.h"

int main(int argc, char **argv)
{
    setlocale(LC_ALL, "");
    /* 一些函数（例如：g_date_set_parse）会根据LC_TIME来确定日期时间的默认解析顺序，因此LC_TIME最好使用系统配置的环境变量，
    否则可能会导致授权码中的过期时间解析错误。例如，如果我们系统的语言环境为en_AU.utf8，当调用g_date_set_parse(gdate, "2050-01-01")时，
    得到的日期是"2001-01-02"，因为该函数解析日期的顺序变为了日-月-年。*/
    setlocale(LC_TIME, "en_US.utf8");
    bindtextdomain(GETTEXT_DOMAIN, LICENSE_LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_DOMAIN, "UTF-8");
    textdomain(GETTEXT_DOMAIN);

    klog_gtk3_init(std::string(), "kylinsec-system", PROJECT_NAME, Glib::path_get_basename(argv[0]));

    Server::LicensePointsManager::global_init();
    std::string server_address = fmt::format("0.0.0.0:{0}", DEFAULT_LISTEN_PORT);
    Server::LicenseImpl service;

    grpc::ServerBuilder builder;
    builder.AddListeningPort(server_address, grpc::InsecureServerCredentials());
    builder.RegisterService(&service);
    std::unique_ptr<grpc::Server> server(builder.BuildAndStart());
    server->Wait();

    Server::LicensePointsManager::global_deinit();

    return 0;
}
